/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.integration.transliteration;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.kemet.impl.grammar.exceptions.KemetAPIException;
import org.kemet.impl.grammar.integration.TransliterationManager;
import org.kemet.tests.TestRoot;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>integration tests : analyze type of translitered sequence - output : list
 * of id (hieroglyph).</b>
 * </p>
 * 
 * 
 * support : uniliteral, biliteral, triliteral, phonetic complement, indicator
 * stating of adverbial predicate proposition (ppa), pronoun, preposition, noun,
 * ideogram
 * 
 * not supported (no transliteration) : determiner
 * 
 * @since 0.5
 * 
 */
public class TestsToHieroglyphs extends TestRoot {
	static final Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>initialize the parameters before to execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	public void init() {
		// nothing
	}

	/**
	 * <p>
	 * <b>indicator ppa + biliteral + uniliteral + uniliteral + uniliteral +
	 * uniliteral + biliteral + uniliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test1() {
		// initialize the data tests
		sequence = "jwnnc3cjm33";
		result = new String[] { "M17", "[G43;Z7]",
				"[M22-M22-N35;M22-M22-N35-N35;M22-M22]", "O29", "D36", "M1",
				"G1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;

		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator ppa + uniliteral + uniliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test2() {
		// initialize the data tests
		sequence = "jwn3"; // 
		result = new String[] { "M17", "[G43;Z7]", "N35", "G1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			String[] res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator ppa + uniliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test3() {
		// initialize the data tests
		sequence = "jwn?"; // 
		result = new String[] { "M17", "[G43;Z7]", "N35" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			String[] res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>triliteral + uniliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test4() {
		// initialize the data tests
		sequence = "cnh)h)"; // 
		result = new String[] { "S34", "Aa1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			String[] res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>??_u.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test5() {
		// initialize the data tests
		sequence = "??_u"; // 

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res = null;

		try {
			res = transliteration.analyze();
			assertEquals(res.length, 0);
		} catch (KemetAPIException e) {
			e.printStackTrace();
			assertTrue(false);
		}

		sequence = "uie"; // 

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			res = transliteration.analyze();
			assertEquals(res.length, 0);
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator ppa + uniliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test6() {
		// initialize the data tests
		sequence = "??jwn"; // 
		result = new String[] { "M17", "[G43;Z7]", "N35" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			String[] res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>biliteral + feminine grammatical kind.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test7() {
		// initialize the data tests
		sequence = "w3.t";
		result = new String[] { "V4", "X1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			String[] res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}

		// initialize the data tests
		sequence = "h.3.t";
		result = new String[] { "M16", "X1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		try {
			String[] res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test8() {
		// initialize the data tests
		sequence = "ms^c";
		result = new String[] { "A12" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram + feminine grammatical kind.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test9() {
		// initialize the data tests
		sequence = "h.w.t";
		result = new String[] { "O6", "X1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + pronoun + uniliteral + uniliteral + feminine
	 * grammatical kind.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test10() {
		// initialize the data tests
		sequence = "jw=fmh_.t";
		result = new String[] { "M17", "[G43;Z7]", "I9", "G17", "F32", "X1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + pronoun "0" + uniliteral + pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test11() {
		// initialize the data tests
		sequence = "jw0m=j";
		result = new String[] { "M17", "[G43;Z7]", "G17",
				"[M17-A1;G7;A1;B1;A40;A50;M17]" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + pronoun + uniliteral + pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test12() {
		// initialize the data tests
		sequence = "jw=jm=t_n";
		result = new String[] { "M17", "[G43;Z7]",
				"[M17-A1;G7;A1;B1;A40;A50;M17]", "G17", "[V13-N35-Z2;V13-N35]" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + demonstrative pronoun + suffix pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test13() {
		// initialize the data tests
		sequence = "jwnn=sn";
		result = new String[] { "M17", "[G43;Z7]",
				"[M22-M22-N35;M22-M22-N35-N35;M22-M22]",
				"[O34-N35-Z2;S29-N35-Z2;S29-N35;O34-N35]" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + suffix pronoun + demonstrative pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test14() {
		// initialize the data tests
		sequence = "jw=nnn";
		result = new String[] { "M17", "[G43;Z7]", "[N35-Z2;N35]",
				"[M22-M22-N35;M22-M22-N35-N35;M22-M22]" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + demonstrative pronoun + uniliteral + uniliteral +
	 * feminine grammatical kind.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test15() {
		// initialize the data tests
		sequence = "jw0mh_.t";
		result = new String[] { "M17", "[G43;Z7]", "G17", "F32", "X1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + noun + preposition + noun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test16() {
		// initialize the data tests
		sequence = "jwb3kh.rnh.t";
		result = new String[] { "M17", "[G43;Z7]", "G29-V31-A1", "D2-Z1",
				"N35-O4-X1-M1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>noun + noun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test17() {
		// initialize the data tests
		sequence = "b3knh.t";
		result = new String[] { "G29-V31-A1", "N35-O4-X1-M1" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>preposition.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test18() {
		// initialize the data tests
		sequence = "h_r";
		result = new String[] { "T28-D21" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_HIEROGLYPHS);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (Exception e) {
			assertTrue(false);
		}
	}
}
